﻿namespace HeuristicLab.Problems.ProgramSynthesis.Base.Erc {
  using System.Collections.Generic;

  using Boolean;
  using Char;
  using Common;
  using Core;
  using Data;
  using Float;
  using FloatVector;
  using Integer;
  using IntegerVector;
  using Parameters;
  using Persistence.Default.CompositeSerializers.Storable;
  using String;
  using StringVector;

  [StorableClass]
  public class ErcOptions : ParameterizedNamedItem, IReadOnlyErcOptions {
    public const string ProbabilityParameterName = "Probability";
    public const string IntegerParameterName = "Integer";
    public const string FloatParameterName = "Float";
    public const string CharParameterName = "Char";
    public const string BooleanParameterName = "Boolean";
    public const string StringParameterName = "String";
    public const string NameParameterName = "Name";
    public const string IntegerVectorParameterName = "IntegerVector";
    public const string FloatVectorParameterName = "FloatVector";
    public const string StringVectorParameterName = "StringVector";

    public ErcOptions() {
      Name = "ERC Options";

      Parameters.Add(new FixedValueParameter<PercentValue>(ProbabilityParameterName));
      Parameters.Add(new ValueParameter<IntegerErcOptions>(IntegerParameterName));
      Parameters.Add(new ValueParameter<FloatErcOptions>(FloatParameterName));
      Parameters.Add(new ValueParameter<BooleanErcOptions>(BooleanParameterName));
      Parameters.Add(new ValueParameter<StringErcOptions>(StringParameterName));
      Parameters.Add(new ValueParameter<CharErcOptions>(CharParameterName));
      Parameters.Add(new ValueParameter<StringErcOptions>(NameParameterName));
      Parameters.Add(new ValueParameter<IntegerVectorErcOptions>(IntegerVectorParameterName));
      Parameters.Add(new ValueParameter<FloatVectorErcOptions>(FloatVectorParameterName));
      Parameters.Add(new ValueParameter<StringVectorErcOptions>(StringVectorParameterName));
    }

    [StorableConstructor]
    public ErcOptions(bool deserializing) : base(deserializing) { }

    public ErcOptions(ErcOptions origin, Cloner cloner) : base(origin, cloner) { }

    public IValueParameter<PercentValue> ErcProbabilityParameter
    {
      get { return (IValueParameter<PercentValue>)Parameters[ProbabilityParameterName]; }
    }

    public double ErcProbability
    {
      get { return ErcProbabilityParameter.Value.Value; }
      set { ErcProbabilityParameter.Value.Value = value; }
    }

    IErcValueItem<int> IReadOnlyErcOptions.IntegerErcOptions { get { return IntegerErcOptions; } }
    IErcValueItem<double> IReadOnlyErcOptions.FloatErcOptions { get { return FloatErcOptions; } }
    IErcValueItem<bool> IReadOnlyErcOptions.BooleanErcOptions { get { return BooleanErcOptions; } }
    IErcValueItem<char> IReadOnlyErcOptions.CharErcOptions { get { return CharErcOptions; } }
    IErcValueItem<string> IReadOnlyErcOptions.StringErcOptions { get { return StringErcOptions; } }
    IErcValueItem<string> IReadOnlyErcOptions.NameErcOptions { get { return NameErcOptions; } }
    IErcValueItem<IReadOnlyList<int>> IReadOnlyErcOptions.IntegerVectorErcOptions { get { return IntegerVectorErcOptions; } }
    IErcValueItem<IReadOnlyList<double>> IReadOnlyErcOptions.FloatVectorErcOptions { get { return FloatVectorErcOptions; } }
    IErcValueItem<IReadOnlyList<string>> IReadOnlyErcOptions.StringVectorErcOptions { get { return StringVectorErcOptions; } }

    public IValueParameter<IntegerErcOptions> IntegerErcOptionsParameter
    {
      get { return (IValueParameter<IntegerErcOptions>)Parameters[IntegerParameterName]; }
    }

    public IntegerErcOptions IntegerErcOptions
    {
      get { return this.IntegerErcOptionsParameter.Value; }
      set { this.IntegerErcOptionsParameter.Value = value; }
    }

    public IValueParameter<FloatErcOptions> FloatErcOptionsParameter
    {
      get { return (IValueParameter<FloatErcOptions>)Parameters[FloatParameterName]; }
    }

    public FloatErcOptions FloatErcOptions
    {
      get { return this.FloatErcOptionsParameter.Value; }
      set { this.FloatErcOptionsParameter.Value = value; }
    }

    /// <summary>
    /// Integer represents ASCII value
    /// </summary>
    public IValueParameter<CharErcOptions> CharErcOptionsParameter
    {
      get { return (IValueParameter<CharErcOptions>)Parameters[CharParameterName]; }
    }

    public CharErcOptions CharErcOptions
    {
      get { return CharErcOptionsParameter.Value; }
      set { CharErcOptionsParameter.Value = value; }
    }

    public IValueParameter<BooleanErcOptions> BooleanErcOptionsParameter
    {
      get { return (IValueParameter<BooleanErcOptions>)Parameters[BooleanParameterName]; }
    }

    public BooleanErcOptions BooleanErcOptions
    {
      get { return BooleanErcOptionsParameter.Value; }
      set { BooleanErcOptionsParameter.Value = value; }
    }

    public IValueParameter<StringErcOptions> StringErcOptionsParameter
    {
      get { return (IValueParameter<StringErcOptions>)Parameters[StringParameterName]; }
    }

    public StringErcOptions StringErcOptions
    {
      get { return StringErcOptionsParameter.Value; }
      set { StringErcOptionsParameter.Value = value; }
    }

    public IValueParameter<StringErcOptions> NameParameter
    {
      get { return (IValueParameter<StringErcOptions>)Parameters[NameParameterName]; }
    }

    public StringErcOptions NameErcOptions
    {
      get { return NameParameter.Value; }
      set { NameParameter.Value = value; }
    }

    public IValueParameter<IntegerVectorErcOptions> IntegerVectorParameter
    {
      get { return (IValueParameter<IntegerVectorErcOptions>)Parameters[IntegerVectorParameterName]; }
    }

    public IntegerVectorErcOptions IntegerVectorErcOptions
    {
      get { return IntegerVectorParameter.Value; }
      set { IntegerVectorParameter.Value = value; }
    }

    public IValueParameter<StringVectorErcOptions> StringVectorParameter
    {
      get { return (IValueParameter<StringVectorErcOptions>)Parameters[StringVectorParameterName]; }
    }

    public StringVectorErcOptions StringVectorErcOptions
    {
      get { return StringVectorParameter.Value; }
      set { StringVectorParameter.Value = value; }
    }

    public IValueParameter<FloatVectorErcOptions> FloatVectorParameter
    {
      get { return (IValueParameter<FloatVectorErcOptions>)Parameters[FloatVectorParameterName]; }
    }

    public FloatVectorErcOptions FloatVectorErcOptions
    {
      get { return FloatVectorParameter.Value; }
      set { FloatVectorParameter.Value = value; }
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ErcOptions(this, cloner);
    }
  }
}
