﻿namespace HeuristicLab.Problems.ProgramSynthesis.Base.Erc.Float {
  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Data;
  using HeuristicLab.Parameters;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

  [StorableClass]
  public class FloatRangeErcValue : WeightedErcValueItem<double> {
    private const string RangeParameterName = "Range";

    public FloatRangeErcValue() {
      this.Parameters.Add(new FixedValueParameter<DoubleRange>(RangeParameterName));
    }

    public FloatRangeErcValue(double start, double end) : this() {
      this.Start = start;
      this.End = end;
    }

    [StorableConstructor]
    public FloatRangeErcValue(bool deseralizing) : base(deseralizing) { }

    public FloatRangeErcValue(FloatRangeErcValue origin, Cloner cloner) : base(origin, cloner) { }

    public IValueParameter<DoubleRange> RangeParameter
    {
      get { return (IValueParameter<DoubleRange>)this.Parameters[RangeParameterName]; }
    }

    public double Start
    {
      get { return this.RangeParameter.Value.Start; }
      set { this.RangeParameter.Value.Start = value; }
    }

    public double End
    {
      get { return this.RangeParameter.Value.End; }
      set { this.RangeParameter.Value.End = value; }
    }

    public override double GetErcValue(IRandom random) {
      return Start < End && IsEnabled ? random.NextDouble() * (End - Start) + Start : 0d;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new FloatRangeErcValue(this, cloner);
    }
  }
}
