﻿namespace HeuristicLab.Problems.ProgramSynthesis.Base.Erc {
  using System.Collections.Generic;
  using System.Linq;
  using Common;
  using Core;
  using Data;
  using Parameters;
  using Persistence.Default.CompositeSerializers.Storable;
  using Random;

  [StorableClass]
  public abstract class VectorConstantsErcValue<T, TValue> : WeightedErcValueItem<IReadOnlyList<T>>
    where T : struct
    where TValue : ValueTypeArray<T> {
    private const string ConstantsParameterName = "Constants";

    private readonly TValue[] arrays; // perf opt: avoid using parameter getter

    protected VectorConstantsErcValue(params TValue[] arrays) : this(true, arrays) { }

    protected VectorConstantsErcValue(bool isEnabled, params TValue[] arrays) {
      this.arrays = arrays;
      IsEnabled = isEnabled;
      Parameters.Add(new ValueParameter<ItemArray<TValue>>(ConstantsParameterName, new ItemArray<TValue>(arrays)));
    }

    [StorableConstructor]
    private VectorConstantsErcValue(bool deserializing) : base(deserializing) { }

    protected VectorConstantsErcValue(VectorConstantsErcValue<T, TValue> origin, Cloner cloner) : base(origin, cloner) { }

    public IValueParameter<ItemArray<TValue>> ConstantsParameter
    {
      get { return (IValueParameter<ItemArray<TValue>>)Parameters[ConstantsParameterName]; }
    }

    public IEnumerable<TValue> Constants
    {
      get { return ConstantsParameter.Value; }
    }

    public override IReadOnlyList<T> GetErcValue(IRandom random) {
      // TODO: remove ToList when ValueTypeArray implements IReadOnlyList
      return arrays.SampleRandom(random).ToList();
    }
  }
}