﻿namespace HeuristicLab.Problems.ProgramSynthesis.Base.Erc {
  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Data;
  using HeuristicLab.Parameters;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

  public abstract class WeightedErcValueItem<T> : ParameterizedNamedItem, IWeightedErcValueItem<T> {
    private const string WeightParameterName = "Weight";

    protected WeightedErcValueItem() : this(false, 1d) { }

    protected WeightedErcValueItem(bool isEnabled, double weigth) {
      IsEnabled = isEnabled;
      Parameters.Add(new FixedValueParameter<DoubleValue>(WeightParameterName, new DoubleValue(weigth)));
    }

    [StorableConstructor]
    protected WeightedErcValueItem(bool deserializing) : base(deserializing) { }

    protected WeightedErcValueItem(WeightedErcValueItem<T> origin, Cloner cloner) : base(origin, cloner) { }

    public IValueParameter<DoubleValue> WeightParameter
    {
      get { return (IValueParameter<DoubleValue>)Parameters[WeightParameterName]; }
    }

    public double Weight
    {
      get { return WeightParameter.Value.Value; }
      set { WeightParameter.Value.Value = value; }
    }

    public bool IsEnabled { get; set; }

    public abstract T GetErcValue(IRandom random);
  }
}
